"""
Python init file for the app editor package. Contains a logger that
the app editor can use to log in isolation from the main app.log
"""

import logging
import logging.handlers as handlers
import datetime


LOG_NAME = "AppEditor"
LOG_LOC = "/store/log/editor.log"

loggers = {}


def get_logger(log_name=LOG_NAME, log_location=LOG_LOC):
    """
    Function to return a logger for the app editor
    """
    if loggers.get(LOG_NAME):
        return loggers.get(LOG_NAME)
    else:
        logger = EditorLogger(LOG_NAME, LOG_LOC)
        loggers.update(({LOG_NAME : logger}))
    return logger


class EditorLogger(object):
    """
    Logger class used by the editor
    """

    def __init__(self, log_name, file_loc):

        self.logger = logging.getLogger(log_name)
        handler = logging.handlers.RotatingFileHandler(file_loc,
                                                       maxBytes=2*1024*1024,
                                                       backupCount=5)
        self.logger.addHandler(handler)
        self.log_level = self.__get_level_by_name('DEBUG')
        self.logger.setLevel(self.log_level)

    def log(self, msg, level="info", *args):
        """
        Function to log a message
        """
        log_timestamp = (datetime.datetime.now().strftime("%b %d %H:%M:%S "))

        log_msg = (log_timestamp + "[%s]" % LOG_NAME +
                   "[%s]" % str(level.upper()) + " %s" % msg)

        self.logger.log(self.__get_level_by_name(level), log_msg, *args)

    def debug(self, msg, *args):
        """
        Debugging logging method
        """
        log_timestamp = (datetime.datetime.now().strftime("%b %d %H:%M:%S "))

        log_msg = (log_timestamp + "[%s]" % LOG_NAME +
                   "[DEBUG]" + " %s" % msg)

        self.logger.debug(log_msg, *args)

    def set_log_level(self, level='info'):
        """
        Sets the current log level for the app editor log
        """
        self.level = self.__get_level_by_name(level)
        self.logger.setLevel(self.level)

    def is_enabled(self, level='debug'):
        """
        Utility function to check if logging is enabled for a given
        level. Defaults to debug
        """
        level_int = self.__get_level_by_name(level)
        return self.logger.isEnabledFor(level_int)

    def __get_level_by_name(self, name):
        """
        Private function to convert a string level into an int
        """
        return logging.getLevelName(name.upper())
