"""
Module containing implementation functions to back the REST API
"""
import os
import shutil
import time
import zipfile
import json
import jsonschema
import requests
import re
from app_editor.py.api.models import File, Folder, FileContents, OrderedResource
from app_editor.py.utils import py_utils
from app_editor import get_logger
from flask import request
from flask import jsonify

MANIFEST_LOCATION = '/app/manifest.json'
EXEMPT_FILES = ['.pyc']
ORDERING_FILE = 'ordering.txt'
logger = get_logger()

class ManifestException(Exception):
    pass

def catalogue_files_with_path(full_path):
    """
    Function to take a full path then convert it into a format
    usable by the catalogue_files function
    """
    if logger.is_enabled():
        msg = ('catalogue_files_with_path(): Cataloguing '
               'using full path [%s]...')
        logger.debug(msg, full_path)

    root_path = os.path.dirname(full_path)
    root_name = os.path.basename(full_path)

    if logger.is_enabled():
        msg = ('catalogue_files_with_path(): Calling catalogue_files '
               'with params [%s] and [%s]...')
        logger.debug(msg, root_path, root_name)

    files = catalogue_files(root_path, root_name)

    if logger.is_enabled():
        msg = ('catalogue_files_with_path(): Finished cataloguing '
               '[%s].')
        logger.debug(msg, full_path)

    return files


def catalogue_files(root_path, root_name):
    """
    Recursive function to catalogue the file tree from a given
    root directory path and name
    @param root_path: path to directory
    @param root_name: name of the directory
    @returns object containing the file tree
    """
    full_path = os.path.join(root_path, root_name)

    if logger.is_enabled():
        msg = "catalogue_files(): Cataloguing from [%s]..."
        logger.debug(msg, full_path)

    if not os.path.exists(full_path):
        raise OSError("Path [%s] cannot be found", full_path)

    folder_obj = Folder(name=root_name, path=full_path)

    is_py_package = False

    for content in os.listdir(full_path):

        file_path = os.path.join(full_path, content)

        if os.path.isdir(file_path):

            if logger.is_enabled():
                msg = "catalogue_files(): Cataloguing directory [%s]..."
                logger.debug(msg, file_path)

            folder_obj.contents.append(catalogue_files(full_path, content))
        # Ignore exempt files (*.pyc for now)
        elif not any(exempt in content for exempt in EXEMPT_FILES):
            if logger.is_enabled():
                msg = "catalogue_files(): Cataloguing file [%s]..."
                logger.debug(msg, file_path)

            if content == '__init__.py':
                is_py_package = True

            tmp_name, file_ext = os.path.splitext(file_path)

            file_obj = File(name=content,
                            path=file_path,
                            file_type=(file_ext if file_ext else 'file'))

            folder_obj.contents.append(file_obj)

    folder_obj.file_type = 'py_package' if is_py_package else 'folder'

    if logger.is_enabled():
        msg = "catalogue_files(): Finished cataloguing [%s]."
        logger.debug(msg, full_path)

    return folder_obj


def get_file_type(file_path):
    """
    Function to return the type for a given file
    @param file_path: path to file/folder
    @return: Str file type
    @raise OSError: raised if the file path does not exist
    """
    if logger.is_enabled():
        msg = "get_file_type(): Getting file type for file [%s]..."
        logger.debug(msg, file_path)

    if not os.path.exists(file_path):
        raise OSError('File path [%s] does not exist.' % file_path)

    file_type = ''

    if os.path.isdir(file_path):
        is_py_package = '__init__.py' in os.listdir(file_path)
        file_type = 'py_package' if is_py_package else 'folder'
    else:
        tmp_name, file_ext = os.path.splitext(file_path)
        file_type = file_ext if file_ext else 'file'

    if logger.is_enabled():
        msg = "get_file_type(): Returning file type [%s] for file [%s]."
        logger.debug(msg, file_type, file_path)

    return file_type


def move_file(src_path, des_path):
    """
    Function to move an item on the file system
    @param src_path: File/folder to be moved
    @param des_path: Destination of the move
    @raise OSError: Raised if des path already exists
    @raise ValueError: Raised if src path does not exist
    @returns File object containing the new file
    """
    if not os.path.exists(src_path):
        error_msg = 'Could not locate resource at path [%s].' % src_path
        raise ValueError(error_msg)
    if os.path.exists(des_path):
        error_msg = 'Resource already exists at path [%s].' % des_path
        raise OSError(error_msg)

    shutil.move(src_path, des_path)
    path, name = os.path.split(des_path)

    file_obj = File(name=name, path=path)
    file_obj.file_type = get_file_type(des_path)

    return file_obj


def rename_file(file_path, new_name):
    """
    function to rename an item on the file system
    @param file_path: path to file to be renamed
    @param new_name: New name for the file
    @throws ValueError: If file path is not found
    @throws OSError: If file already exists with given name in given folder
    @returns File object containing the new file
    """
    if logger.is_enabled:
        msg = "rename_file(): Renaming file [%s] to [%s]..."
        logger.debug(msg, file_path, new_name)

    if not os.path.exists(file_path):
        raise ValueError('File path [%s] does not exist.' % file_path)

    file_dir = os.path.dirname(file_path)
    new_path = os.path.join(file_dir, new_name)

    if os.path.exists(new_path):
        raise OSError('New file path [%s] already exists.' % new_path)

    os.rename(file_path, new_path)

    file_obj = File(name=new_name, path=new_path)
    file_obj.file_type = get_file_type(new_path)

    if logger.is_enabled():
        msg = "rename_file(): Renamed file [%s] to [%s]."
        logger.debug(msg, file_path, new_name)

    return file_obj


def create_file(file_path, is_dir=False):
    """
    Function to create a file or directory
    @param file_path: path to new file
    @param is_dir: Designates if it is a directory or not
    @return: Object containing the new file/folder
    @raises OSError: Raised if the file path already exists
    """
    if logger.is_enabled():
        msg = "create_file(): Creating file (is_dir=%s) [%s]..."
        logger.debug(msg, str(is_dir), file_path)

    if os.path.exists(file_path):
        raise OSError('File path [%s] already exists.' % file_path)

    result_obj = None

    if is_dir:
        os.makedirs(file_path)
        dir_name = os.path.basename(file_path)
        result_obj = Folder(name=dir_name, path=file_path, file_type='folder')
    else:
        os.mknod(file_path)
        file_name = os.path.basename(file_path)
        file_type = get_file_type(file_path)
        result_obj = File(name=file_name, path=file_path, file_type=file_type)

    if logger.is_enabled():
        msg = "create_file(): Created file (is_dir=%s) [%s]."
        logger.debug(msg, is_dir, file_path)

    return result_obj


def copy_file(src_path, des_path):
    """
    Function to copy a file
    @param src_path: path to existing file
    @param des_path: path to file copy destination
    @raise ValueError: Raised if src path does not exist
    @raise OSError: Raised if des path already exists
    @returns: Object containing the new file copy
    """
    if logger.is_enabled():
        msg = "copy_file(): Copying file [%s] to [%s]..."
        logger.debug(msg, src_path, des_path)

    if not os.path.exists(src_path):
        raise ValueError('File path [%s] does not exist.' % src_path)
    if os.path.exists(des_path):
        raise OSError('File already exists at path [%s].', des_path)

    shutil.copy2(src_path, des_path)
    path, name = os.path.split(des_path)

    file_obj = File(name=name, path=des_path)
    file_obj.file_type = get_file_type(des_path)

    return file_obj


def delete_file(file_path):
    """
    Function to delete a file from the file system
    @param file_path: path to file/folder to be deleted
    @raises OSError: Raised if the file could not be found
    """
    if logger.is_enabled():
        logger.debug('delete_file(): Deleting file [%s]...', file_path)

    if not os.path.exists(file_path):
        raise OSError('File path [%s] does not exist.' % file_path)

    if os.path.isdir(file_path):
        shutil.rmtree(file_path)
    else:
        os.remove(file_path)

    if logger.is_enabled():
        logger.debug('delete_file(): Deleted file [%s].', file_path)


def get_file_contents(file_path):
    """
    Function to get the contents of a file
    @param file_path: path to the file to be read
    @returns: FileContents object containing the files contents
    @raises ValueError: Raised if the file can not be found
    """
    if logger.is_enabled():
        msg = "get_file_contents(): Returning file contents for [%s]..."
        logger.debug(msg, file_path)

    if not os.path.exists(file_path):
        raise ValueError('File path [%s] does not exist.' % file_path)

    content = ''

    with open(file_path, 'r') as open_file:
        for lines in open_file:
            content = content + lines.decode('utf-8')

    contents_obj = FileContents(path=file_path, contents=content)

    if logger.is_enabled():
        msg = "get_file_contents(): Returning contents [%s] for file [%s]."
        logger.debug(msg, content, file_path)

    return contents_obj


def write_file_contents(file_path, contents):
    """
    Function to write contents to a file
    @param file_path: path to file to be written to
    @param file_contents: FileContents object containing the contents
    @raises ValueError: Raised if the file can not be found
    """
    if logger.is_enabled():
        msg = "write_file_contents(): Writing contents [%s] to file [%s]..."
        logger.debug(msg, contents, file_path)

    if not os.path.exists(file_path):
        raise ValueError('File path [%s] does not exist.' % file_path)

    with open(file_path, 'w+') as target_file:
        target_file.write(contents.encode('utf-8'))

    if logger.is_enabled():
        msg = "write_file_contents(): Contents written to file [%s]."
        logger.debug(msg, file_path)

# Directory Utils

def create_tmp_dir_str():
    """
    Function to create a tmp directory name from a
    time stamp.
    @return: Tmp directory name from current timestamp
    """

    current_time = int(time.time())
    dir_name = "/tmp/tmp_app_" + str(current_time)

    if logger.is_enabled():
        msg = "create_tmp_dir_str: Created tmp directory name [%s]."
        logger.debug(msg, dir_name)

    return dir_name


def copy_dir(src_dir, target_dir):
    """
    Function to copy one directory to another
    @param src_dir: Directory to be copied
    @param target_dir: Directory to be copied to
    """
    if logger.is_enabled():
        msg = "copy_dir: Copying directory [%s] to target [%s]..."
        logger.debug(msg, src_dir, target_dir)

    shutil.copytree(src_dir, target_dir)

    if logger.is_enabled():
        msg = "copy_dir: Copied directory [%s] to target [%s]."
        logger.debug(msg, src_dir, target_dir)


def delete_dir(dir_path):
    """
    Wrapper function to delete a directory
    @param dir_path: Path to directory to be deleted
    """
    if logger.is_enabled():
        msg = "delete_dir: Deleting directory [%s]."
        logger.debug(msg, dir_path)

    shutil.rmtree(dir_path)

    if logger.is_enabled():
        msg = "delete_dir: directory [%s] deleted."
        logger.debug(msg, dir_path)


def has_dir(dir_path):
    """
    Wrapper function to check if a directory exists
    @param dir_path: Path to directory
    @returns: True if directory exists, false otherwise
    """
    if logger.is_enabled():
        msg = "has_dir: Checking if directory [%s] exists."
        logger.debug(msg, dir_path)

    has_dir = os.path.isdir(dir_path)

    if logger.is_enabled():
        msg = "has_dir: Returning [%s]."
        logger.debug(msg, dir_path)

    return has_dir

# Zip Utils


def zip_dir(dir_path, zip_path=None):
    """
    Function to produce a zip file from a directory's contents
    @param dir_path: Path to directory to be zipped
    @param zip_path: Path to zip file containing the archive
    """
    if logger.is_enabled():
        msg = "zip_dir: Zipping directory [%s] to zip file [%s]..."
        logger.debug(msg, dir_path, zip_path)

    if not os.path.isdir(dir_path):
        raise OSError("dir_path argument must point to a directory. "
                      "'%s' does not." % dir_path)
    if not zip_path:
        zip_path = dir_path + '.zip'

    parent_dir, dir_to_zip = os.path.split(dir_path)

    def trim_path(path):
        archive_path = path.replace(parent_dir, "", 1)

        if parent_dir:
            archive_path = archive_path.replace(os.path.sep, "", 1)

        archive_path = archive_path.replace(dir_to_zip + os.path.sep, "",
                                            1)
        return os.path.normcase(archive_path)

    output_zip = zipfile.ZipFile(zip_path,
                                 'w',
                                 compression=zipfile.ZIP_DEFLATED)

    for arch_path, dir_names, files_names in os.walk(dir_path):
        for file_name in files_names:
            file_path = os.path.join(arch_path, file_name)
            output_zip.write(file_path, trim_path(file_path))

    output_zip.close()

    if logger.is_enabled():
        msg = "zip_dir: Finished zipping file [%s]."
        logger.debug(msg, zip_path)


def get_manifest(manifest_loc):
    """
    Function to read and return the application's manifest
    @param manifest_loc: Path to app manifest
    @return Manifest JSON read from file
    @raises OSError if specified manifest location cannot be found
    """
    if logger.is_enabled():
        msg = "get_manifest(): Retrieving manifest [%s]..."
        logger.debug(msg, manifest_loc)

    if not os.path.exists(manifest_loc):
        raise OSError('File path [%s] does not exist.' % manifest_loc)

    with open(manifest_loc, 'r') as manifest:

        manifest_json = json.load(manifest)

    # if logger.is_enabled():
    #     msg = "get_manifest(): Returning manifest [%s]."
    #     logger.debug(msg, str(manifest_json))
    if logger.is_enabled():
        logger.debug("returning manifest...")

    return manifest_json


def write_manifest(manifest_loc, manifest_json):
    """
    Function to write the manifest at the specified location
    @param manifest_loc: Path to app manifest
    @return: Reference to manifest json written to file
    @raises OSError if specified manifest location cannot be found
    """
    if logger.is_enabled():
        msg = "write_manifest(): Writing manifest [%s] to [%s]..."
        logger.debug(msg, str(manifest_json), manifest_loc)

    if not os.path.exists(manifest_loc):
        raise OSError('File path [%s] does not exist.' % manifest_loc)

    with open(manifest_loc, 'w+') as manifest:
        manifest.write(json.dumps(manifest_json))

    if logger.is_enabled():
        msg = "write_manifest(): Manifest written to [%s]."
        logger.debug(msg, manifest_loc)

    return manifest_json


def get_sdk_path():
    return os.path.dirname(os.path.dirname(os.path.dirname(os.path.realpath(__file__))))


def get_manifest_schema():
    sdk_path = get_sdk_path()
    static_path = os.path.join(sdk_path, "static")
    resources_path = os.path.join(static_path, "resources")
    validation_path = os.path.join(resources_path, "validation")
    schema_json_path = os.path.join(validation_path, "manifest-schema.json")
    if logger.is_enabled():
        msg = "get_manifest_schema(): Using schema located at [%s]."
        logger.debug(msg, schema_json_path)
    schema = open(schema_json_path).read()
    return schema


def validate_manifest(manifest_data):
    schema = get_manifest_schema()
    manifest_json = json.loads(manifest_data)

    try:
        v = jsonschema.Draft4Validator(json.loads(schema))
        errors = sorted(v.iter_errors(manifest_json), key=lambda e: e.path)
        if errors:
            err_str=''
            for error in errors:
                err_str += '\n'
                for err_thing in error.path:
                    err_str += '[' + str(err_thing) + ']'
                if error.path:
                    err_str += ': '
                err_str += error.message

            raise ManifestException('Schema violations:' + err_str)
    except (ValueError, TypeError) as err:
        raise ManifestException('Your manifest.json is not valid JSON:\n' + str(err))

    current_app_id = __get_app_id()
    current_console_ip = __get_console_address()

    if str(manifest_json.get("app_id")) != current_app_id:
        raise ManifestException('Protected Property: cannot modify app_id [was '
                                + str(current_app_id) + ' now ' + str(manifest_json.get("app_id")) + ']')

    if manifest_json.get("console_ip") != current_console_ip:
        raise ManifestException('Protected Property: cannot modify console_ip [was '
                                + str(current_console_ip) + ' now ' + str(manifest_json.get("console_ip")) + ']')

def get_ordered_packages(dir_path):
    """
    Function to get the rpm/pip packages used by the Application
    @param dir_path: Path to package directory
    @returns List containing OrderedResource objects for
             the specified directory path
    """
    if logger.is_enabled():
        msg = "get_ordered_packages(): Getting packages from directory [%s]..."
        logger.debug(msg, dir_path)

    packages = []

    if not os.path.exists(dir_path):

        if logger.is_enabled():
            msg = ("get_ordered_packages(): No directory [%s] found. "
                   "returning [].")
            logger.debug(msg, dir_path)

        return packages

    ordering_txt = os.path.join(dir_path, ORDERING_FILE)

    if os.path.exists(ordering_txt):

        if logger.is_enabled():
            msg = ("get_ordered_packages(): Found ordering file [%s]. "
                   "Reading order...")
            logger.debug(msg, ordering_txt)

        with open(ordering_txt, 'r') as ordering:

            pos = 0

            for line in ordering:
                if logger.is_enabled():
                    msg = ("get_ordered_packages(): Found package "
                           "[%s] in position [%s]...")
                    logger.debug(msg, line, str(pos))

                packages.append(OrderedResource(name=line, position=pos))
                pos += 1
    else:
        if logger.is_enabled():
            msg = ("get_ordered_packages(): No ordering file [%s] found "
                       "but found packages. Defaulting position to [0]...")
            logger.debug(msg, ordering_txt)

        for package in os.listdir(dir_path):

            if logger.is_enabled():
                msg = "get_ordered_packages(): Found package [%s]..."
                logger.debug(msg, package)

            packages.append(OrderedResource(name=package))

    if logger.is_enabled():
        msg = "get_ordered_packages(): Retrieved [%s] packages."
        logger.debug(msg, str(len(packages)))

    return packages


def change_package_order(dir_path, package_name, pos):
    """
    Function to change the ordering of packages
    @param dir_path: Path to package directory
    @param package_name: Name of package to be reordered
    @param pos: new position for the package
    @returns OrderedResource list for the specified directory
    @raise OSError: Raised if the dir_path or package could not be found
    """
    if logger.is_enabled():
        msg = ("change_package_order(): Changing [%s] "
               "order for package [%s] to position [%s]...")
        logger.debug(msg, dir_path, package_name, str(pos))

    if not os.path.exists(dir_path):
        raise OSError('Directory path [%s] does not exist.' % dir_path)

    if package_name not in os.listdir(dir_path):
        raise OSError('Package [{0}] could not be located in directory [{1}].'
                      .format(package_name, dir_path))

    ordering_txt = os.path.join(dir_path, ORDERING_FILE)
    has_ordering = os.path.isfile(ordering_txt)
    # If no ordering file exists then create one
    if not has_ordering:

        if logger.is_enabled():
            msg = ("change_package_order(): No [%s] found. "
                   "Creating it and defaulting [%s] to position [0]..")
            logger.debug(msg, ordering_txt, package_name)

        with open(ordering_txt, 'r+') as ordering_file:

            ordering_file.write(package_name + '\n')

            for each in os.listdir(dir_path):
                if each == package_name:
                    continue
                else:
                    ordering_file.write(each + '\n')
    else:
        if logger.is_enabled():
            msg = ("change_package_order(): Found [%s]. Reinserting [%s] at "
                   "position [%s]...")
            logger.debug(msg, ordering_txt, package_name, str(pos))

        with open(ordering_txt, 'w+') as ordering_file:

            lines = ordering_file.readlines()
            lines_size = len(lines) - 1

            if pos > lines_size:
                raise ValueError('Position [{0}] is invalid for list of size '
                                 '[{1}].'.format(pos, lines_size))

            lines.remove(package_name)
            lines.insert(pos, package_name)

            for each in lines:
                ordering_file.write(each)

    new_order = get_ordered_packages(dir_path)

    if logger.is_enabled():
        msg = "change_package_order(): Returning new order [%s]."
        logger.debug(msg, str(py_utils.to_json_dict(new_order)))

    return new_order


def delete_package(dir_path, package_name):
    """
    Function to delete a pip/rpm package
    @param dir_path: Path to the directory of the package
    @param package_name: Package to be deleted
    """
    if logger.is_enabled():
        msg = "delete_package(): Deleting package [%s] from directory [%s]..."
        logger.debug(msg, package_name, dir_path)

    if not os.path.exists(dir_path):
        raise OSError('Directory path [%s] does not exist.' % dir_path)

    if package_name not in os.listdir(dir_path):
        raise OSError('Package [{0}] could not be located in directory [{1}].'
                      .format(package_name, dir_path))

    os.remove(package_name)

    ordering_txt = os.path.join(dir_path, ORDERING_FILE)
    has_ordering = os.path.isfile(ordering_txt)

    if has_ordering:

        if logger.is_enabled():
            msg = ("delete_package(): [%s] detected. "
                   "Deleting entry for package [%s]..")
            logger.debug(msg, ordering_txt, package_name)

        with open(ordering_txt, 'w+') as ordering_file:
            lines = ordering_file.readLines()
            for each in lines:
                if package_name not in each:
                    ordering_file.write(each)

    if logger.is_enabled():
        msg = "delete_package(): Successfully deleted package [%s] from [%s]."
        logger.debug(msg, package_name, dir_path)


def deploy_application_development():
    """
    Function to deploy an application in development mode enabling the editor
    functionality to still be available after the upgrade. The process involves
    incrementing the version number, zipping up the source code and then posting
    to the corresponding qradar app upgrade endpoint
    @returns: QRadar response indicating result of upgrade request
    """

    if logger.is_enabled():
        logger.debug("deploy_application_development(): Beginning application deploy in development mode.")

    if logger.is_enabled():
        msg = "deploy_application_development(): Updating version number in manifest..."
        logger.debug(msg)

    manifest = get_manifest(MANIFEST_LOCATION)
    updated_manifest = __increment_version_number(manifest)
    manifest = write_manifest(MANIFEST_LOCATION, updated_manifest)

    tmp_dir = create_tmp_dir_str()

    target_app_loc = tmp_dir + '/app'

    if logger.is_enabled():
        msg = "deploy_application_development(): Copying app source to directory [%s]..."
        logger.debug(msg, target_app_loc)

    copy_dir('/app', target_app_loc)

    target_editor_loc = tmp_dir + '/app_editor'

    if logger.is_enabled():
        msg = "deploy_application_development(): Copying editor source to directory [%s]..."
        logger.debug(msg, target_editor_loc)

    copy_dir('/app_editor', target_editor_loc)

    target_manifest_loc = tmp_dir + '/manifest.json'

    if logger.is_enabled():
        msg = "deploy_application_development(): Copying app manifest to directory [%s]..."
        logger.debug(msg, target_manifest_loc)

    copy_file(MANIFEST_LOCATION, target_manifest_loc)

    editor_run_loc = tmp_dir + '/run.py'

    if logger.is_enabled():
        msg = "deploy_application_development(): Copying editor run.py to directory [%s]..."
        logger.debug(msg, editor_run_loc)

    copy_file('/run.py', editor_run_loc)

    tmp_zip = tmp_dir + '.zip'

    if logger.is_enabled():
        msg = "deploy_application_development(): Writing app to zip file [%s]..."
        logger.debug(msg, tmp_zip)

    zip_dir(tmp_dir, tmp_zip)

    response = __put_app_zip(tmp_zip)
    response_data = json.loads(response.get_data())

    if not response_data.get('success'):
        if logger.is_enabled():
            msg = "deploy_application_development(): __put_app_zip didn't return success...."
            logger.debug(msg)
        if response_data.get('message'):
            raise ValueError(response_data.get('message'))
        raise ValueError("No response content returned from QRadar.")

    if logger.is_enabled():
        msg = "deploy_application_development(): App built and posted to QRadar...."
        logger.debug(msg)

    if logger.is_enabled():
        msg = "deploy_application_development(): Removing files [%s] and [%s]...."
        logger.debug(msg, tmp_dir, tmp_zip)

    delete_dir(tmp_dir)
    delete_file(tmp_zip)

    return response


def deploy_application_live():
    """
    Function to deploy an application in live mode which will strip out all
    editor references. The process involves incrementing the version number,
    zipping up the source code while performing removal of the editor and then
    posting to the corresponding qradar app upgrade endpoint
    @returns: QRadar response indicating result of upgrade request
    """

    if logger.is_enabled():
        logger.debug("deploy_application_live(): Beginning application deploy in live mode.")

    if logger.is_enabled():
        msg = "deploy_application_live(): Updating version number in manifest..."
        logger.debug(msg)

    manifest = get_manifest(MANIFEST_LOCATION)
    updated_manifest = __increment_version_number(manifest)
    write_manifest(MANIFEST_LOCATION, updated_manifest)

    tmp_zip_path = __create_live_app_zip()

    response = __put_app_zip(tmp_zip_path)
    response_data = json.loads(response.get_data())

    if not response_data.get('success'):
        if logger.is_enabled():
            msg = "deploy_application_live(): __put_app_zip didn't return success...."
            logger.debug(msg)
        if response_data.get('message'):
            raise ValueError(response_data.get('message'))
        raise ValueError("No response content returned from QRadar.")

    if logger.is_enabled():
        msg = "deploy_application_live(): App built and posted to QRadar...."
        logger.debug(msg)

    if logger.is_enabled():
        msg = "deploy_application_live(): Removing zip file [%s]..."
        logger.debug(msg, tmp_zip_path)

    delete_file(tmp_zip_path)

    editor_log_path = '/store/log/editor.log'
    if logger.is_enabled():
        msg = "deploy_application_live(): Removing log file [%s]...."
        logger.debug(msg, editor_log_path)

    try:
        delete_file(editor_log_path)
    except OSError:
        pass

    return response


def export_application_zip(version_override=None):
    """
    Small helper to call the __create_live_app_zip() function for creating a
    zip of the application currently being modified in the editor
    @returns: the name of the app being exported and the path to the zip file
    """

    if version_override:
        if logger.is_enabled():
            msg = "export_application_zip(): Updating manifest version for export...."
            logger.debug(msg)
        manifest = get_manifest(MANIFEST_LOCATION)
        manifest['version'] = version_override
        write_manifest(MANIFEST_LOCATION, manifest)

    tmp_zip_path = __create_live_app_zip(sanitise_manifest=True)
    app_name = __get_app_name().replace(' ', '_')

    return app_name, tmp_zip_path


def __put_app_zip(zip_file_path):
    """
    Function to put a zip file to the application_creation_task
    endpoint
    @param zip_file_path: Path to zip to be posted
    @returns: Response object containing response from the server
    """
    with open(zip_file_path, 'r') as zip_file:
        app_id = __get_app_id()
        endpoint = 'api/gui_app_framework/applications/%s' % app_id
        headers = {'Content-Type': 'application/zip'}
        qradar_response = None
        rv = {}

        fullURL = "https://" + __get_console_address() + "/" + endpoint

        if logger.is_enabled():
            msg = "__put_app_zip(): Putting zip [%s] to endpoint [%s]..."
            logger.debug(msg, zip_file_path, fullURL)

        headers = __get_tokens(headers)

        try:
            qradar_response = requests.put(fullURL, params=None, headers=headers, verify=False,
                                auth=None, data=zip_file, json=None, timeout=3)
        except Exception as e:
            logger.debug("__put_app_zip(): request returned exception [%s]" % str(e))
            rv['success'] = False
            rv['message'] = str(e)
            response = jsonify(rv)
            return response

        if logger.is_enabled():
            msg = "__put_app_zip(): Zip [%s] put. Endpoint returned status code [%s]"

        rv['success'] = qradar_response.status_code == 202

        try:
            json = qradar_response.json()
            rv['payload'] = json
            rv['payload']['response_url'] = qradar_response.url
            rv['message'] = json['status']
        except ValueError:
            rv['message'] = 'Invalid content returned in response from QRadar.'
            logger.debug("__put_app_zip(): Response from qradar had no json to decode")

        response = jsonify(rv)
        response.status_code = qradar_response.status_code

    return response


def __increment_version_number(manifest_data):
    """
    Function to increment the version number in the manifest
    @param manifest_data: The current contents of the manifest
    @returns: The updated manifest contents
    """
    manifest = manifest_data
    current_version = str(manifest['version'])

    if logger.is_enabled():
        msg = "__increment_version_number(): Current version number [%s]"
        logger.debug(msg, current_version)

    pos = current_version.rfind('.')

    if pos != -1:
        version_prefix = current_version[pos+1:]
        if str.isdigit(version_prefix):
            version_prefix = str(int(version_prefix)+1)
            manifest['version'] = current_version[:pos+1] + version_prefix
            if logger.is_enabled():
                msg = "__increment_version_number(): New version number [%s]"
                logger.debug(msg, manifest['version'])
            return manifest

    manifest['version'] = current_version + '.1'
    if logger.is_enabled():
        msg = "__increment_version_number(): New version number [%s]"
        logger.debug(msg, manifest['version'])
    return manifest


def __remove_editor_from_manifest(manifest_data):
    """
    Function to remove information related to the editor from a manifest
    @param manifest_data: The current contents of the manifest
    @returns: The updated manifest contents
    """
    manifest = manifest_data
    editor_id = 'editorArea_' + manifest['name']
    editor_id = editor_id.replace(' ', '_')

    updated_areas = []
    for area in manifest['areas']:
        if area['id'] != editor_id:
            updated_areas.append(area)

    manifest['areas'] = updated_areas

    return manifest


def __sanitise_manifest_for_export(manifest_data):
    """
    Function to remove unnecessary information from a manifest in preparation for exporting
    @param manifest_data: The current contents of the manifest
    @returns: The updated manifest contents
    """
    manifest = manifest_data

    for manifest_item in ['console_ip', 'app_id', 'docker_host']:
        if manifest.get(manifest_item):
            if logger.is_enabled():
                msg = " __sanitise_manifest_for_export(): removing [%s] from manifest"
                logger.debug(msg, manifest_item)
            del manifest[manifest_item]

    return manifest


def get_version():
    manifest = get_manifest(MANIFEST_LOCATION)
    version = ''
    if 'version' not in manifest:
        version = '1.0'
    else:
        version = manifest['version']
    return version


def __get_console_address():
    manifest = get_manifest(MANIFEST_LOCATION)
    console_ip = ''
    if 'console_ip' not in manifest:
        console_ip = '127.0.0.1'
    else:
        console_ip = manifest['console_ip']
    return console_ip


def __get_app_name():
    manifest = get_manifest(MANIFEST_LOCATION)
    app_name = 'None'
    if 'name' in manifest.keys():
        app_name = str(manifest["name"])
    return app_name


def __get_app_id():
    manifest = get_manifest(MANIFEST_LOCATION)
    app_id = '0'
    if 'app_id' in manifest.keys():
        app_id = str(manifest["app_id"])
    return app_id


def __acquire_SEC_token():
    return request.cookies.get('SEC')


def __acquire_QRadarCSRF_token():
    return request.cookies.get('QRadarCSRF')


def __get_tokens(headers, version=None):
    if headers is None:
        headers = {}
    if 'SEC' not in headers:
        headers['SEC'] = __acquire_SEC_token()
    if 'QRadarCSRF' not in headers:
        headers['QRadarCSRF'] = __acquire_QRadarCSRF_token()
    if version is not None:
        headers['Version'] = version
    return headers


def __create_live_app_zip(sanitise_manifest=False):
    """
    Function to create a zip file containing the app currently being modified
    via the editor. The word 'live' implies removing all traces of the editor
    @param sanitise_manifest: If True, the app_id, console_ip and docker_host
    will be stripped out of the manifest before creating the zip
    @returns: The path of the newly created zip file
    """

    if logger.is_enabled():
        logger.debug("__create_live_app_zip(): Beginning creation of live app zip.")

    tmp_dir = create_tmp_dir_str()

    target_app_loc = tmp_dir + '/app'

    if logger.is_enabled():
        msg = "__create_live_app_zip(): Copying app source to directory [%s]..."
        logger.debug(msg, target_app_loc)

    copy_dir('/app', target_app_loc)

    target_manifest_loc = tmp_dir + '/manifest.json'

    if logger.is_enabled():
        msg = "__create_live_app_zip(): Copying app manifest to directory [%s]..."
        logger.debug(msg, target_manifest_loc)

    copy_file(MANIFEST_LOCATION, target_manifest_loc)

    if logger.is_enabled():
        msg = "__create_live_app_zip(): Removing editor area from manifest..."
        logger.debug(msg)

    manifest = get_manifest(target_manifest_loc)
    updated_manifest = __remove_editor_from_manifest(manifest)

    if sanitise_manifest:
        if logger.is_enabled():
            msg = "__create_live_app_zip(): Sanitising manifest for export..."
            logger.debug(msg)
        updated_manifest = __sanitise_manifest_for_export(updated_manifest)

    write_manifest(target_manifest_loc, updated_manifest)

    if logger.is_enabled():
        msg = "__create_live_app_zip() Removing original manifest from app folder..."
        logger.debug(msg)

    delete_file(tmp_dir + '/app/manifest.json')

    tmp_zip = tmp_dir + '.zip'

    if logger.is_enabled():
        msg = "__create_live_app_zip(): Writing app to zip file [%s]..."
        logger.debug(msg, tmp_zip)

    zip_dir(tmp_dir, tmp_zip)
    delete_dir(tmp_dir)

    return tmp_zip
