"""
Module containing the REST API endpoints used by the app editor
"""
import json
from app_editor import get_logger
from app_editor.py.api import api_impl
from app_editor.py.utils import py_utils
from flask import request
from flask import Blueprint
from flask import make_response
from flask import send_file
from flask import jsonify


class ApiException(Exception):
    status_code = 500

    def __init__(self, message, status_code=None, payload=None):
        Exception.__init__(self)
        self.message = message
        if status_code is not None:
            self.status_code = status_code
        self.payload = payload

    def to_dict(self):
        rv = {}
        if self.payload is not None:
            rv['payload'] = self.payload
        rv['message'] = self.message
        rv['success'] = False
        return rv


def generate_api_response(message, status_code=None, payload=None):
    status = 200

    if status_code is not None:
        status = status_code

    rv = {}
    if payload is not None:
        rv['payload'] = payload

    rv['message'] = message
    rv['success'] = True

    response = jsonify(rv)
    response.status_code = status

    return response


api_blueprint = Blueprint('editor_apis', __name__, url_prefix='/api')
logger = get_logger()

if logger.is_enabled():
    logger.debug('rest_api: Creating editor REST API endpoints.')


@api_blueprint.route('/files', methods=['GET', 'POST', 'DELETE'])
def files():
    if request.method == 'GET':

        root_dir = request.args.get('root_dir')

        if logger.is_enabled():
            logger.debug('files(): GET request retrieved with root_dir [%s].'
                         % root_dir)

        if not root_dir:
            error_msg = 'Request is missing root_dir query parameter'
            raise ApiException(error_msg, 400)

        try:
            files = api_impl.catalogue_files_with_path(root_dir)

            if logger.is_enabled():
                logger.debug('files(): GET request successful. '
                             'Returning files.')

            return generate_api_response("File List Successful", 200, py_utils.to_json_dict(files))

        except OSError as ex:
            error_msg = ex.errno % ex.strerror
            raise ApiException(error_msg, 404)
        except Exception as ex:
            raise ApiException(str(ex), 500)

    elif request.method == 'POST':

        path = request.args.get('path')

        if not path:
            error_msg = "Request is missing path query parameter"
            raise ApiException(error_msg, 400)

        if 'new_name' in request.args and 'new_path' in request.args:
            error_msg = ('New_name and new_path are mutually '
                         'exclusive query parameters')
            raise ApiException(error_msg, 400)

        if 'new_name' in request.args:
            new_name = request.args.get('new_name')

            try:
                renamed_file = api_impl.rename_file(path, new_name)
                return generate_api_response("File Renamed Successfully [%s]" % path, 200, py_utils.to_json_dict(renamed_file))

            except ValueError as ex:
                raise ApiException(str(ex), 404)
            except Exception as ex:
                raise ApiException(str(ex), 500)

        elif 'new_path' in request.args:

            new_path = request.args.get('new_path')
            copy = request.args.get('copy')

            if copy:
                try:
                    copied_file = api_impl.copy_file(path, new_path)
                    return generate_api_response("File Copied Successfully [%s]" % path, 200, py_utils.to_json_dict(copied_file))

                except ValueError as ex:
                    raise ApiException(str(ex), 404)
                except Exception as ex:
                    raise ApiException(str(ex), 500)

            else:
                try:
                    moved_file = api_impl.move_file(path, new_path)
                    return generate_api_response("File Moved Successfully [%s]" % path, 200, py_utils.to_json_dict(moved_file))

                except ValueError as ex:
                    raise ApiException(str(ex), 404)
                except Exception as ex:
                    raise ApiException(str(ex), 500)

        else:
            try:
                is_dir = request.args.get('is_dir') if 'is_dir' \
                        in request.args else None
                is_dir = is_dir == 'true'

                new_file = api_impl.create_file(path, is_dir)
                return generate_api_response("File/Folder Created Successfully [%s]" % path, 200, py_utils.to_json_dict(new_file))

            except Exception as ex:
                raise ApiException(str(ex), 500)

    elif request.method == 'DELETE':

        path = request.args.get('path')

        if not path:
            error_msg = "Request is missing path query parameter"
            raise ApiException(error_msg, 400)
        try:
            api_impl.delete_file(path)
            return generate_api_response("File Deleted Successfully [%s]" % path, 200)
        except OSError as ex:
            raise ApiException(str(ex), 404)


@api_blueprint.route('/files/contents', methods=['GET', 'POST'])
def files_contents():
    path = request.args.get('path')

    if not path:
        error_msg = "Request is missing path query parameter"
        raise ApiException(error_msg, 500)

    if request.method == 'GET':

        try:
            file_contents = api_impl.get_file_contents(path)
            return generate_api_response("File Contents Retrieved Successfully [%s]" % path, 200, py_utils.to_json_dict(file_contents))

        except ValueError as ex:
            raise ApiException(str(ex), 404)
        except Exception as ex:
            raise ApiException(str(ex), 500)

    elif request.method == 'POST':

        request_json = request.json

        if 'contents' in request_json:
            file_contents = request_json['contents']
        else:
            error_msg = "Missing contents in request JSON"
            raise ApiException(error_msg, 400)

        if (path == '/app/manifest.json'):
            try:
                api_impl.validate_manifest(file_contents)
            except api_impl.ManifestException as ex:
                raise ApiException(str(ex), 400)
            except Exception as ex:
                raise ApiException(str(ex), 500)

        try:
            api_impl.write_file_contents(path, file_contents)
            return generate_api_response("File Contents Written Successfully [%s]" % path, 200)
        except ValueError as ex:
            raise ApiException(str(ex), 404)


@api_blueprint.route('/deploy', methods=['POST'])
def deploy_application():
    is_live = request.args.get('is_live') if 'is_live' \
            in request.args else None
    is_live = is_live == 'true'

    if is_live:
        try:
            response = api_impl.deploy_application_live()
            return response
        except OSError as ex:
            raise ApiException(str(ex), 404)
        except Exception as ex:
            raise ApiException(str(ex), 500)
    else:
        try:
            response = api_impl.deploy_application_development()
            return response
        except OSError as ex:
            raise ApiException(str(ex), 404)
        except Exception as ex:
            raise ApiException(str(ex), 500)


@api_blueprint.route('/export', methods=['GET'])
def export_application():
    export_format = request.args.get('format')
    new_version = request.args.get('version')

    if not export_format:
        error_msg = "Request is missing export_format query parameter"
        raise ApiException(error_msg, 400)
    else:
        if export_format == 'zip':
            try:
                if new_version:
                    app_name, zip_path = api_impl.export_application_zip(new_version)
                else:
                    app_name, zip_path = api_impl.export_application_zip()
                return send_file(zip_path,
                                 as_attachment=True,
                                 attachment_filename=app_name + '.zip')
            except Exception as ex:
                raise ApiException(str(ex), 500)


@api_blueprint.route('/version', methods=['GET'])
def get_application_version():
    try:
        version = api_impl.get_version()
        return generate_api_response('App Version Retrieved Successfully', 200, version);
    except Exception as ex:
        raise ApiException(str(ex), 500)
