'''
Module containing unit tests for the api implementations
'''
import unittest
import os
from app_editor.py.api import api_impl


class TestFileAPIImplementation(unittest.TestCase):

    def setUp(self):
        self.tmp_file = 'tmp_test.txt'
        self.tmp_file_new = 'tmp_test_new.txt'
        self.tmp_dir = 'tmp_dir'
        self.tmp_dir_new = 'tmp_dir_new'

    def test_catalogue_files_with_path(self):

        full_path = 'src/app_template/app'
        app_editor = api_impl.catalogue_files_with_path(full_path)

        has_templates = False
        has_init = False
        has_views = False

        for each in app_editor.contents:
            if each.name == 'templates' and each.file_type == 'folder':
                has_templates = True
            if each.name == '__init__.py' and each.file_type == '.py':
                has_init = True
            if each.name == 'views.py' and each.file_type == '.py':
                has_views = True

        self.assertTrue((has_templates and
                         has_init and
                         has_views))

    def test_catalogue_files(self):

        root_path = 'src/app_template'
        root_dir = 'app'

        app_editor = api_impl.catalogue_files(root_path, root_dir)

        has_templates = False
        has_init = False
        has_views = False

        for each in app_editor.contents:
            if each.name == 'templates' and each.file_type == 'folder':
                has_templates = True
            if each.name == '__init__.py' and each.file_type == '.py':
                has_init = True
            if each.name == 'views.py' and each.file_type == '.py':
                has_views = True

        self.assertTrue((has_templates and
                         has_init and
                         has_views))

    def test_catalogue_files_invalid_path(self):

        invalid_name = "invalid"
        invalid_path = "/"
        self.assertRaises(OSError, api_impl.catalogue_files,
                          invalid_path,
                          invalid_name)

    def test_get_file_type(self):

        test_file = 'src/app_template/app_editor/editor.py'
        test_folder = 'src/app_template/app_editor/templates'
        test_py_package = 'src/app_template/app_editor'

        file_type_file = api_impl.get_file_type(test_file)
        file_type_folder = api_impl.get_file_type(test_folder)
        file_type_py_package = api_impl.get_file_type(test_py_package)

        is_file = file_type_file == '.py'
        is_folder = file_type_folder == 'folder'
        is_py_package = file_type_py_package == 'py_package'

        self.assertTrue((is_file and is_folder and is_py_package))

    def test_get_file_type_invalid_path(self):

        invalid_path = "/tmp/invalid.oops"
        self.assertRaises(OSError, api_impl.get_file_type, invalid_path)

    def test_create_file_file(self):

        test_file = '/tmp/' + self.tmp_file
        result = api_impl.create_file(test_file)

        self.assertEqual(result.path, test_file)
        self.assertEqual(result.file_type, '.txt')
        self.assertTrue(os.path.isfile(test_file))

        os.remove(test_file)

    def test_create_file_dir(self):

        test_dir = '/tmp/' + self.tmp_dir
        result = api_impl.create_file(test_dir, is_dir=True)

        self.assertEqual(result.path, test_dir)
        self.assertEqual(result.file_type, 'folder')
        self.assertTrue(os.path.isdir(test_dir))

        os.rmdir(test_dir)

    def test_create_file_already_exists(self):

        test_file = '/tmp/test.txt'
        os.mknod(test_file)
        self.assertRaises(OSError, api_impl.create_file, test_file)
        os.remove(test_file)

    def test_move_file_file(self):

        test_file = '/tmp/test.txt'
        os.mknod(test_file)
        des_dir = '/tmp/test_dir'
        file_des = des_dir + '/test.txt'

        os.mkdir(des_dir)
        api_impl.move_file(test_file, file_des)

        move_success = (not os.path.exists(test_file) and
                        os.path.exists(file_des))

        self.assertTrue(move_success)
        os.remove(file_des)
        os.rmdir(des_dir)

    def test_move_file_dir(self):

        test_dir = '/tmp/test_dir1'
        os.mkdir(test_dir)
        tmp2_dir = '/tmp/tmp2'
        des_dir = tmp2_dir + '/test_dir2'
        api_impl.move_file(test_dir, des_dir)

        move_success = (not os.path.exists(test_dir) and
                        os.path.exists(des_dir))

        self.assertTrue(move_success)
        os.rmdir(des_dir)
        os.rmdir(tmp2_dir)

    def test_move_file_invalid_src_path(self):

        test_dir = '/tmp/test_invalid'
        des_dir = '/tmp/test_dir'
        self.failUnlessRaises(ValueError,
                              api_impl.move_file,
                              test_dir,
                              des_dir)

    def test_move_file_des_exists(self):

        test_dir = '/tmp/' + self.tmp_dir
        os.mkdir(test_dir)

        des_root = '/tmp/' + self.tmp_dir_new
        os.mkdir(des_root)

        des_dir = des_root + '/' + self.tmp_dir
        os.mkdir(des_dir)

        self.failUnlessRaises(OSError, api_impl.move_file, test_dir, des_dir)

        os.rmdir(des_dir)
        os.rmdir(des_root)
        os.rmdir(test_dir)

    def test_rename_file_file(self):

        test_file = '/tmp/' + self.tmp_file
        os.mknod(test_file)
        new_name = self.tmp_file_new
        result = api_impl.rename_file(test_file, new_name)
        expected_path = '/tmp/' + new_name

        self.assertEquals(result.path, expected_path)
        self.assertFalse(os.path.exists(test_file))
        self.assertTrue(os.path.exists(expected_path))

        os.remove(expected_path)

    def test_rename_file_dir(self):

        test_dir = '/tmp/' + self.tmp_dir
        os.mkdir(test_dir)
        new_dir = self.tmp_dir_new
        result = api_impl.rename_file(test_dir, new_dir)
        expected_path = '/tmp/' + new_dir

        self.assertEqual(result.path, expected_path)
        self.assertFalse(os.path.exists(test_dir))
        self.assertTrue(os.path.exists(expected_path))

        os.rmdir(expected_path)

    def test_rename_file_invalid_path(self):

        invalid_path = "/tmp/invalid.oops"
        new_name = "invalid.txt"
        self.assertRaises(ValueError,
                          api_impl.rename_file,
                          invalid_path,
                          new_name)

    def test_rename_file_name_exists(self):

        existing_file = '/tmp/tmp_file.py'
        existing_name = 'tmp_file.py'
        os.mknod(existing_file)

        file_target = '/tmp/tmp2_file.py'
        os.mknod(file_target)

        self.assertRaises(OSError,
                          api_impl.rename_file,
                          file_target,
                          existing_name)

        os.remove(existing_file)
        os.remove(file_target)

    def test_copy_file(self):

        test_file = '/tmp/test.txt'
        os.mknod(test_file)
        des_dir = '/tmp/test_dir'
        file_des = des_dir + '/test_copy.txt'

        os.mkdir(des_dir)
        api_impl.copy_file(test_file, file_des)

        copy_success = (os.path.exists(test_file) and
                        os.path.exists(file_des))

        self.assertTrue(copy_success)
        os.remove(test_file)
        os.remove(file_des)
        os.rmdir(des_dir)

    def test_copy_file_invalid_src_path(self):

        test_dir = '/tmp/test_invalid'
        des_dir = '/tmp/test_dir'
        self.failUnlessRaises(ValueError,
                              api_impl.copy_file,
                              test_dir,
                              des_dir)

    def test_copy_file_des_exits(self):

        test_file = '/tmp/test.txt'
        os.mknod(test_file)
        des_dir = '/tmp/test_dir'
        os.mkdir(des_dir)
        existing_file = des_dir + '/test_copy.txt'
        os.mknod(existing_file)

        self.failUnlessRaises(OSError, api_impl.copy_file, test_file, existing_file)

        os.remove(test_file)
        os.remove(existing_file)
        os.rmdir(des_dir)

    def test_delete_file(self):

        test_file = '/tmp/' + self.tmp_file
        os.mknod(test_file)
        api_impl.delete_file(test_file)
        self.assertFalse(os.path.exists(test_file))

    def test_delete_dir(self):

        test_dir = '/tmp/' + self.tmp_dir
        os.mkdir(test_dir)
        api_impl.delete_file(test_dir)
        self.assertFalse(os.path.exists(test_dir))

    def test_delete_file_invalid(self):

        test_file = '/tmp/invalid.txt'
        self.assertRaises(OSError, api_impl.delete_file, test_file)

    def test_get_file_contents(self):

        test_str = "Hello World"
        test_file = '/tmp/' + self.tmp_file
        os.mknod(test_file)

        with open(test_file, 'w+') as file_test:
            file_test.write(test_str)

        contents_result = api_impl.get_file_contents(test_file)
        contents = contents_result.contents

        self.assertEqual(test_str, contents)
        os.remove(test_file)

    def test_write_file_contents(self):

        test_str = "Hello World"
        test_file = '/tmp/' + self.tmp_file
        os.mknod(test_file)

        api_impl.write_file_contents(test_file, test_str)

        with open(test_file, 'r') as file_test:
            result = file_test.readline()

        self.assertEqual(test_str, result)
        os.remove(test_file)

    def test_validate_manifest(self):

        test_manifest_data = '''{
            "name":"Application Development",
            "log_level":"debug",
            "console_ip": "9.21.119.123",
            "app_id": 1001,
            "description":"Application for installing other applications in Development-mode.",
            "version":"1.0.0",
            "uuid":"ae1c84d0-be94-11e5-a837-0800200c9a66",
            "configuration_pages":[
                    {
                        "text":"Develop Applications",
                        "description":"Application Development",
                        "url":"bootstrapper",
                        "required_capabilities":["ADMIN"],
                        "icon":null
                    }
                ]
            }'''
        try:
            api_impl.validate_manifest(test_manifest_data)
        except api_impl.ManifestException as e:
            self.fail(e)
        except Exception as e:
            self.fail(str(e))

    def test_validate_manifest_invalid(self):

        test_manifest_data = '''{
            "name":"Application Development",
            "log_level":"debug",
            "version":"1.0.0",
            "configuration_pages":[
                {
                    "text":"Develop Applications",
                    "description":"Application Development",
                    "url":"bootstrapper",
                    "required_capabilities":["ADMIN"],
                    "icon":null
                }
                ]
        }'''

        self.assertRaises(api_impl.ManifestException, api_impl.validate_manifest, test_manifest_data)
